#include <avr/io.h>
#include <util/delay.h>
#include "ShiftRegister.h"
/***************************************

Configure Connections

****************************************/

#define HC595_PORT   PORTC
#define HC595_DDR    DDRC

#define HC595_DS_POS PC0      //Data pin (DS) pin location

#define HC595_SH_CP_POS PC2      //Shift Clock (SH_CP) pin location
#define HC595_ST_CP_POS PC1      //Store Clock (ST_CP) pin location
//======
#define HC595_PORT1   PORTC
#define HC595_DDR1    DDRC

#define HC595_DS_POS1 PC4      //Data pin (DS) pin location

#define HC595_SH_CP_POS1 PC6      //Shift Clock (SH_CP) pin location
#define HC595_ST_CP_POS1 PC5      //Store Clock (ST_CP) pin location
//======
#define HC595_PORT2   PORTB
#define HC595_DDR2    DDRB

#define HC595_DS_POS2 PB7      //Data pin (DS) pin location

#define HC595_SH_CP_POS2 PB5      //Shift Clock (SH_CP) pin location
#define HC595_ST_CP_POS2 PB6      //Store Clock (ST_CP) pin location
//======
#define HC595_PORT3   PORTD
#define HC595_DDR3    DDRD

#define HC595_DS_POS3 PD3      //Data pin (DS) pin location

#define HC595_SH_CP_POS3 PD5      //Shift Clock (SH_CP) pin location
#define HC595_ST_CP_POS3 PD4      //Store Clock (ST_CP) pin location

/***************************************
Configure Connections ***ENDS***
****************************************/

//Initialize HC595 System

void HC595Init()
{
   //Make the Data(DS), Shift clock (SH_CP), Store Clock (ST_CP) lines output
   HC595_DDR|=((1<<HC595_SH_CP_POS)|(1<<HC595_ST_CP_POS)|(1<<HC595_DS_POS));
}


//Low level macros to change data (DS)lines
#define HC595DataHigh() (HC595_PORT|=(1<<HC595_DS_POS))

#define HC595DataLow() (HC595_PORT&=(~(1<<HC595_DS_POS)))

//Sends a clock pulse on SH_CP line
void HC595Pulse()
{
   //Pulse the Shift Clock

   HC595_PORT|=(1<<HC595_SH_CP_POS);//HIGH

   HC595_PORT&=(~(1<<HC595_SH_CP_POS));//LOW

}

//Sends a clock pulse on ST_CP line
void HC595Latch()
{
   //Pulse the Store Clock

   HC595_PORT|=(1<<HC595_ST_CP_POS);//HIGH
   _delay_loop_1(1);

   HC595_PORT&=(~(1<<HC595_ST_CP_POS));//LOW
   _delay_loop_1(1);
}


/*

Main High level function to write a single byte to
Output shift register 74HC595.

Arguments:
   single byte to write to the 74HC595 IC

Returns:
   NONE

Description:
   The byte is serially transfered to 74HC595
   and then latched. The byte is then available on
   output line Q0 to Q7 of the HC595 IC.

*/
void HC595Write(uint8_t data)
{
   //Send each 8 bits serially

   //Order is MSB first
   for(uint8_t i=0;i<8;i++)
   {
      //Output the data on DS line according to the
      //Value of MSB
      if(data & 0b10000000)
      {
         //MSB is 1 so output high

         HC595DataHigh();
      }
      else
      {
         //MSB is 0 so output high
         HC595DataLow();
      }

      HC595Pulse();  //Pulse the Clock line
      data=data<<1;  //Now bring next bit at MSB position

   }

   //Now all 8 bits have been transferred to shift register
   //Move them to output latch at one
   HC595Latch();
}

//====
//Initialize HC595 System

void HC595Init1()
{
   //Make the Data(DS), Shift clock (SH_CP), Store Clock (ST_CP) lines output
   HC595_DDR1|=((1<<HC595_SH_CP_POS1)|(1<<HC595_ST_CP_POS1)|(1<<HC595_DS_POS1));
}


//Low level macros to change data (DS)lines
#define HC595DataHigh1() (HC595_PORT1|=(1<<HC595_DS_POS1))

#define HC595DataLow1() (HC595_PORT1&=(~(1<<HC595_DS_POS1)))

//Sends a clock pulse on SH_CP line
void HC595Pulse1()
{
   //Pulse the Shift Clock

   HC595_PORT1|=(1<<HC595_SH_CP_POS1);//HIGH

   HC595_PORT1&=(~(1<<HC595_SH_CP_POS1));//LOW

}

//Sends a clock pulse on ST_CP line
void HC595Latch1()
{
   //Pulse the Store Clock

   HC595_PORT1|=(1<<HC595_ST_CP_POS1);//HIGH
   _delay_loop_1(1);

   HC595_PORT1&=(~(1<<HC595_ST_CP_POS1));//LOW
   _delay_loop_1(1);
}


/*

Main High level function to write a single byte to
Output shift register 74HC595.

Arguments:
   single byte to write to the 74HC595 IC

Returns:
   NONE

Description:
   The byte is serially transfered to 74HC595
   and then latched. The byte is then available on
   output line Q0 to Q7 of the HC595 IC.

*/
void HC595Write1(uint8_t data)
{
   //Send each 8 bits serially

   //Order is MSB first
   for(uint8_t i=0;i<8;i++)
   {
      //Output the data on DS line according to the
      //Value of MSB
      if(data & 0b10000000)
      {
         //MSB is 1 so output high

         HC595DataHigh1();
      }
      else
      {
         //MSB is 0 so output high
         HC595DataLow1();
      }

      HC595Pulse1();  //Pulse the Clock line
      data=data<<1;  //Now bring next bit at MSB position

   }

   //Now all 8 bits have been transferred to shift register
   //Move them to output latch at one
   HC595Latch1();
}
//====
//Initialize HC595 System

void HC595Init2()
{
   //Make the Data(DS), Shift clock (SH_CP), Store Clock (ST_CP) lines output
   HC595_DDR2|=((1<<HC595_SH_CP_POS2)|(1<<HC595_ST_CP_POS2)|(1<<HC595_DS_POS2));
}


//Low level macros to change data (DS)lines
#define HC595DataHigh2() (HC595_PORT2|=(1<<HC595_DS_POS2))

#define HC595DataLow2() (HC595_PORT2&=(~(1<<HC595_DS_POS2)))

//Sends a clock pulse on SH_CP line
void HC595Pulse2()
{
   //Pulse the Shift Clock

   HC595_PORT2|=(1<<HC595_SH_CP_POS2);//HIGH

   HC595_PORT2&=(~(1<<HC595_SH_CP_POS2));//LOW

}

//Sends a clock pulse on ST_CP line
void HC595Latch2()
{
   //Pulse the Store Clock

   HC595_PORT2|=(1<<HC595_ST_CP_POS2);//HIGH
   _delay_loop_1(1);

   HC595_PORT2&=(~(1<<HC595_ST_CP_POS2));//LOW
   _delay_loop_1(1);
}


/*

Main High level function to write a single byte to
Output shift register 74HC595.

Arguments:
   single byte to write to the 74HC595 IC

Returns:
   NONE

Description:
   The byte is serially transfered to 74HC595
   and then latched. The byte is then available on
   output line Q0 to Q7 of the HC595 IC.

*/
void HC595Write2(uint8_t data)
{
   //Send each 8 bits serially

   //Order is MSB first
   for(uint8_t i=0;i<8;i++)
   {
      //Output the data on DS line according to the
      //Value of MSB
      if(data & 0b10000000)
      {
         //MSB is 1 so output high

         HC595DataHigh2();
      }
      else
      {
         //MSB is 0 so output high
         HC595DataLow2();
      }

      HC595Pulse2();  //Pulse the Clock line
      data=data<<1;  //Now bring next bit at MSB position

   }

   //Now all 8 bits have been transferred to shift register
   //Move them to output latch at one
   HC595Latch2();
}
//====
//Initialize HC595 System

void HC595Init3()
{
   //Make the Data(DS), Shift clock (SH_CP), Store Clock (ST_CP) lines output
   HC595_DDR3|=((1<<HC595_SH_CP_POS3)|(1<<HC595_ST_CP_POS3)|(1<<HC595_DS_POS3));
}


//Low level macros to change data (DS)lines
#define HC595DataHigh3() (HC595_PORT3|=(1<<HC595_DS_POS3))

#define HC595DataLow3() (HC595_PORT3&=(~(1<<HC595_DS_POS3)))

//Sends a clock pulse on SH_CP line
void HC595Pulse3()
{
   //Pulse the Shift Clock

   HC595_PORT3|=(1<<HC595_SH_CP_POS3);//HIGH

   HC595_PORT3&=(~(1<<HC595_SH_CP_POS3));//LOW

}

//Sends a clock pulse on ST_CP line
void HC595Latch3()
{
   //Pulse the Store Clock

   HC595_PORT3|=(1<<HC595_ST_CP_POS3);//HIGH
   _delay_loop_1(1);

   HC595_PORT3&=(~(1<<HC595_ST_CP_POS3));//LOW
   _delay_loop_1(1);
}


/*

Main High level function to write a single byte to
Output shift register 74HC595.

Arguments:
   single byte to write to the 74HC595 IC

Returns:
   NONE

Description:
   The byte is serially transfered to 74HC595
   and then latched. The byte is then available on
   output line Q0 to Q7 of the HC595 IC.

*/
void HC595Write3(uint8_t data)
{
   //Send each 8 bits serially

   //Order is MSB first
   for(uint8_t i=0;i<8;i++)
   {
      //Output the data on DS line according to the
      //Value of MSB
      if(data & 0b10000000)
      {
         //MSB is 1 so output high

         HC595DataHigh3();
      }
      else
      {
         //MSB is 0 so output high
         HC595DataLow3();
      }

      HC595Pulse3();  //Pulse the Clock line
      data=data<<1;  //Now bring next bit at MSB position

   }

   //Now all 8 bits have been transferred to shift register
   //Move them to output latch at one
   HC595Latch3();
}
/*

Simple Delay function approx 0.5 seconds

*/

void Wait()
{
   for(uint8_t i=0;i<30;i++)
   {
      _delay_loop_2(0);
   }
}
