#include "Usart.h"

#define BAUD_PRESCALE (((F_CPU / (baud_rate * 16UL))) - 1)

/*
 * Functie de initializare a controllerului USART
 */
void USART_init(unsigned long baud_rate)
{
	UBRR0L = BAUD_PRESCALE;
	UBRR0H = (BAUD_PRESCALE >> 8);
	UCSR0C |= (1 << UCSZ00) | (1 << UCSZ01);
	UCSR0B |= (1 << TXEN0) | (1 << RXEN0);
}

/*
 * Functie ce transmite un caracter prin USART
 *
 * @param data - caracterul de transmis
 */
void USART_transmit(char data)
{
    UDR0 = data;
    while(!(UCSR0A & (1 << UDRE0)));
}

/*
 * Functie ce primeste un caracter prin USART
 *
 * @return - caracterul primit
 */
char USART_receive()
{
    while(!(UCSR0A & (1 << RXC0)));
    return UDR0;
}

/*
 * Functie ce transmite un sir de caractere prin USART
 *
 * @param data - sirul (terminat cu '\0') de transmis
 */
void USART_print(const char *data)
{
    while(*data != '\0')
        USART_transmit(*data++);
}
